package com.github.jchanghong

import cn.hutool.core.io.FileUtil
import cn.hutool.core.lang.TypeReference
import cn.hutool.core.util.StrUtil
import cn.hutool.db.dialect.DriverUtil
import cn.hutool.json.JSONUtil
import com.baomidou.mybatisplus.annotation.DbType
import com.baomidou.mybatisplus.annotation.IdType
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils
import com.baomidou.mybatisplus.core.toolkit.StringPool
import com.baomidou.mybatisplus.generator.AutoGenerator
import com.baomidou.mybatisplus.generator.InjectionConfig
import com.baomidou.mybatisplus.generator.config.*
import com.baomidou.mybatisplus.generator.config.builder.ConfigBuilder
import com.baomidou.mybatisplus.generator.config.converts.PostgreSqlTypeConvert
import com.baomidou.mybatisplus.generator.config.po.TableInfo
import com.baomidou.mybatisplus.generator.config.rules.DateType
import com.baomidou.mybatisplus.generator.config.rules.FileType
import com.baomidou.mybatisplus.generator.config.rules.IColumnType
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy
import com.baomidou.mybatisplus.generator.engine.AbstractTemplateEngine
import com.baomidou.mybatisplus.generator.engine.FreemarkerTemplateEngine
import org.gradle.api.Project
import java.io.File
import java.util.*

data class CodeCItem(
        var author: String = "jiangchanghong",
        var dbtype: String = "postgres",
        var includeTablesRegexp: String = "tableName",
        var moduleName: String = "check",
        var parentPackage: String = "com.test",
        var password: String = "12312345",
        var url: String = "jdbc:postgresql://localhost:5432/postgres",
        var username: String = "postgres",
        var includeController: Boolean = true,
        var idType: IdType = IdType.AUTO
)

// 演示例子，执行 main 方法控制台输入模块表名回车自动生成对应项目目录中
object CodeGenerator {
    val scanner = Scanner(System.`in`)

    //    /**
//     *
//     *
//     * 读取控制台内容
//     *
//     */
//    fun scanner(tip: String, project: Project, default: String): String {
//       project.logger.quiet("please input $tip (input  . use default : $default):")
//        val ipt = scanner.next()
//        if (StringUtils.isNotBlank(ipt)&&ipt!=".") {
//            return ipt
//        }
//        else{
//            return default
//        }
//    }
    fun main2(args: Array<String>, project: Project) {
        val file = File(project.projectDir, "auto.json")
        if (!file.exists()) {
            project.logger.quiet("file : ${file.absolutePath}  not exists !!")
            project.logger.quiet("content example is:")
            project.logger.quiet(JSONUtil.toJsonStr(listOf(CodeCItem())))
            return
        }
        val toBean = try {
            JSONUtil.toBean(JSONUtil.parse(file.readText()), object : TypeReference<List<CodeCItem>>() {}, true)
        } catch (e: Exception) {
            null
        }
        if (toBean.isNullOrEmpty()) {
            project.logger.quiet("content example is:")
            project.logger.quiet(JSONUtil.toJsonStr(listOf(CodeCItem())))
            return
        }
        project.logger.quiet(toBean.size.toString())
        toBean.forEach {
            main3(args, project, it)
        }
    }

    @JvmStatic
    fun main3(args: Array<String>, project: Project, jch: CodeCItem) {
        project.logger.quiet(JSONUtil.toJsonStr(jch))
        // 代码生成器
        val autoGenerator = AutoGenerator()
        val projectPath = project.projectDir.absolutePath

        autoGenerator.globalConfig = GlobalConfig().apply {
            outputDir = "$projectPath\\src\\main\\kotlin"
            author = jch.author
            isKotlin = true
            dateType = DateType.TIME_PACK
            isSwagger2 = true
            idType = jch.idType
            isOpen = false
            isFileOverride = true
        }
        autoGenerator.dataSource = DataSourceConfig().apply {
            // 数据源配置
            url = jch.url
            // setSchemaName("public");
            driverName = DriverUtil.identifyDriver(url)
            username = jch.username
            password = jch.password
            val dbtype = jch.dbtype
            this.dbType = when (dbtype) {
                "mysql" -> DbType.MYSQL
                "oracle" -> DbType.ORACLE
                else -> DbType.POSTGRE_SQL
            }
            typeConvert = PostgreSqlTypeConvert2
        }


        // 包配置
        autoGenerator.packageInfo = PackageConfig().apply {
//            moduleName = scanner("模块名")
            moduleName = jch.moduleName
            parent = jch.parentPackage
//            pathInfo= mapOf("test" to "test")
        }
//      注入配置
        // 自定义配置
        autoGenerator.cfg = object : InjectionConfig() {
            override fun initMap() {
                // to do nothing
                val tableInfoList = autoGenerator.config.tableInfoList
                tableInfoList.forEach {
                    project.logger.quiet(it.name)
                    it.entityName = "Do${it.entityName}"
                }
                tableInfoList.forEach { it.serviceName = "Auto${it.serviceName}" }
                tableInfoList.forEach { it.serviceImplName = "Auto${it.serviceImplName}" }
                tableInfoList.forEach { it.mapperName = "Auto${it.mapperName}" }
                tableInfoList.forEach { it.controllerName = "Auto${it.controllerName}" }
                val pathInfo = autoGenerator.config.pathInfo
                for ((k, v) in pathInfo) {
//                    project.logger.quiet("$k ${v}")
                }
//                mapper_path
                map = mapOf(
                        "mybatisInsertBatchPG" to mybatisInsertBatchPG(tableInfoList),
                        "ispg" to (autoGenerator.dataSource.dbType == DbType.POSTGRE_SQL)
                )
            }

            private fun mybatisInsertBatchPG(tableInfoList: List<TableInfo>): Map<String, String> {
                return tableInfoList.map {
                    val names = it.fields.filter { it.isKeyFlag == false }
                    val trim = """
                                        INSERT INTO ${it.name} (
                                         ${names.joinToString { it.name }}
                                        )
                   VALUES
                   <foreach collection="list" separator="," item="a">
                   ( ${names.joinToString { "#{a.${it.propertyName}${if ("json" !in it.type) "}" else ",jdbcType = OTHER,typeHandler = com.github.jchanghong.autoconfig.db.mybatis.PGJsonTypeHandler}"}" }}
                   )
                   </foreach>
                   on conflict do nothing
                                    """.trimIndent().trim()
                    it.name to (if (autoGenerator.dataSource.dbType == DbType.POSTGRE_SQL) trim else "")
                }.toMap()
            }
        }.apply {
            // 自定义输出配置
            val focList: MutableList<FileOutConfig> = ArrayList()
            // 自定义配置会被优先输出
            focList.add(object : FileOutConfig("/templates/mapper.xml.ftl") {
                override fun outputFile(tableInfo: TableInfo): String {
                    // 自定义输出文件名 ， 如果你 Entity 设置了前后缀、此处注意 xml 的名称会跟着发生变化！！
                    return ("$projectPath/src/main/resources/mapper/${autoGenerator.packageInfo.moduleName}/${
                        tableInfo.entityName
                    }Mapper${StringPool.DOT_XML}")
                }
            })
            focList.add(object : FileOutConfig("/templates/entityvo.kt.ftl") {
                override fun outputFile(tableInfo: TableInfo): String {
                    // 自定义输出文件名 ， 如果你 Entity 设置了前后缀、此处注意 xml 的名称会跟着发生变化！！
                    return ("$projectPath/src/main/kotlin/${
                        (autoGenerator.packageInfo.parent).replace(
                                ".",
                                "/"
                        )
                    }/entityvo/${tableInfo.entityName}VO.kt")
                }
            })
            fileCreate = object : IFileCreate {
                override fun isCreate(cb: ConfigBuilder, fileType: FileType, filePath: String?): Boolean {
//                    // 判断自定义文件夹是否需要创建
////                    checkDir("调用默认方法创建的目录，自定义目录用");
//                    val b = !FileUtil.exist(filePath!!)
//                    if (fileType == FileType.ENTITY) {
//                        // 已经生成 mapper 文件判断存在，不想重新生成返回 false
//                        return true
//                    }
//                    // 允许生成模板文件
//                    return b
                    // 自定义判断
                    // 自定义判断
                    // 全局判断【默认】
                    // 全局判断【默认】
                    val file = File(filePath)
                    val exist = file.exists()
                    if (!exist) {
                        file.parentFile.mkdirs()
                        return true
                    }
//                    文件存在以后
                    return fileType == FileType.ENTITY && cb.globalConfig.isFileOverride
                }
            }
            fileOutConfigList = focList
        }
        autoGenerator.template = TemplateConfig().apply {
            // 配置模板
            // 配置自定义输出模板
            //指定自定义模板路径，注意不要带上.ftl/.vm, 会根据使用的模板引擎自动识别
//           setEntity("/templates/entity.kt")
            // templateConfig.setService();
            // templateConfig.setController();
            xml = null
            if (!jch.includeController) {
                project.logger.quiet("not create controller")
                this.controller = null
            }
        }

        // 策略配置
        autoGenerator.strategy = StrategyConfig().apply {
            naming = NamingStrategy.underline_to_camel
            columnNaming = NamingStrategy.underline_to_camel
//        superEntityClass = "你自己的父类实体,没有就不用设置!"
            isEntityLombokModel = false
            isRestControllerStyle = true
            this.isEntitySerialVersionUID = true
            this.isEntityTableFieldAnnotationEnable = true
            // 公共父类
//        superControllerClass = "你自己的父类控制器,没有就不用设置!"
            // 写于父类中的公共字段
//            setSuperEntityColumns("id")
            this.isEnableSqlFilter = false
            val toTypedArray = StrUtil.splitTrim(jch.includeTablesRegexp, ",").toTypedArray()
            setInclude(*toTypedArray)
            isControllerMappingHyphenStyle = true
//            setTablePrefix(pc.moduleName + "_")
        }
        autoGenerator.templateEngine = FreemarkerTemplateEngine2(project)
        autoGenerator.execute()
        val pathInfo = autoGenerator.config.pathInfo
        for ((k, v) in pathInfo) {
            if ("mapper_path" == k) {
                val file = File(v)
                for (filei in file.listFiles()) {
                    FileUtil.move(filei, File(filei.absolutePath.replace("\\kotlin\\", "\\java\\", false)), true)

                }
            }
        }
        project.logger.quiet("The generated code is complete! ! ! ! ! ! ! ! ! ! Only overwrite entity by default")
    }
}

object PostgreSqlTypeConvert2 : PostgreSqlTypeConvert() {
    override fun processTypeConvert(config: GlobalConfig?, fieldType: String?): IColumnType {

        if ("jsonb" in fieldType!! || "json" in fieldType!!) {
            return object : IColumnType {
                override fun getPkg(): String {
                    return "cn.hutool.json.JSONObject"
                }

                override fun getType(): String {
                    return "JSONObject"
                }
            }
        }
        return super.processTypeConvert(config, fieldType)
    }
}

class FreemarkerTemplateEngine2(val project: Project) : FreemarkerTemplateEngine() {
    override fun batchOutput(): AbstractTemplateEngine {
        try {
            val tableInfoList = configBuilder.tableInfoList
            for (tableInfo in tableInfoList) {
                val objectMap = getObjectMap(tableInfo)
                val pathInfo = configBuilder.pathInfo
                val template = configBuilder.template
                // 自定义内容
                val injectionConfig = configBuilder.injectionConfig
                if (null != injectionConfig) {
                    injectionConfig.initTableMap(tableInfo)
                    objectMap["cfg"] = injectionConfig.map
                    val focList = injectionConfig.fileOutConfigList
                    if (CollectionUtils.isNotEmpty(focList)) {
                        for (foc in focList) {
                            if (isCreate(FileType.OTHER, foc!!.outputFile(tableInfo))) {
                                val outputFile = foc.outputFile(tableInfo)
                                writerFile(objectMap, foc.templatePath, outputFile)
                                project.logger.quiet(outputFile)
                            }
                        }
                    }
                }
                // Mp.java
                val entityName = tableInfo.entityName
                if (null != entityName && null != pathInfo[ConstVal.ENTITY_PATH]) {
                    val entityFile = String.format(
                            pathInfo[ConstVal.ENTITY_PATH].toString() + File.separator + "%s" + suffixJavaOrKt(),
                            entityName
                    )
                    if (isCreate(FileType.ENTITY, entityFile)) {
                        writerFile(
                                objectMap,
                                templateFilePath(template.getEntity(configBuilder.globalConfig.isKotlin)),
                                entityFile
                        )
                        project.logger.quiet(entityFile)
                    }
                }
                // MpMapper.java
                if (null != tableInfo.mapperName && null != pathInfo[ConstVal.MAPPER_PATH]) {
                    val mapperFile = String.format(
                            pathInfo[ConstVal.MAPPER_PATH].toString() + File.separator + tableInfo.mapperName + ".java",
                            entityName
                    )
                    if (isCreate(FileType.MAPPER, mapperFile)) {
                        writerFile(objectMap, templateFilePath(template.mapper), mapperFile)
                        project.logger.quiet(mapperFile)
                    }
                }
                // MpMapper.xml
                if (null != tableInfo.xmlName && null != pathInfo[ConstVal.XML_PATH]) {
                    val xmlFile = String.format(
                            pathInfo[ConstVal.XML_PATH].toString() + File.separator + tableInfo.xmlName + ConstVal.XML_SUFFIX,
                            entityName
                    )
                    if (isCreate(FileType.XML, xmlFile)) {
                        writerFile(objectMap, templateFilePath(template.xml), xmlFile)
                        project.logger.quiet(xmlFile)
                    }
                }
                // IMpService.java
                if (null != tableInfo.serviceName && null != pathInfo[ConstVal.SERVICE_PATH]) {
                    val serviceFile = String.format(
                            pathInfo[ConstVal.SERVICE_PATH].toString() + File.separator + tableInfo.serviceName + suffixJavaOrKt(),
                            entityName
                    )
                    if (isCreate(FileType.SERVICE, serviceFile)) {
                        writerFile(objectMap, templateFilePath(template.service), serviceFile)
                        project.logger.quiet(serviceFile)
                    }
                }
                // MpServiceImpl.java
                if (null != tableInfo.serviceImplName && null != pathInfo[ConstVal.SERVICE_IMPL_PATH]) {
                    val implFile = String.format(
                            pathInfo[ConstVal.SERVICE_IMPL_PATH].toString() + File.separator + tableInfo.serviceImplName + suffixJavaOrKt(),
                            entityName
                    )
                    if (isCreate(FileType.SERVICE_IMPL, implFile)) {
                        writerFile(objectMap, templateFilePath(template.serviceImpl), implFile)
                        project.logger.quiet(implFile)
                    }
                }
                // MpController.java
                if (null != tableInfo.controllerName && null != pathInfo[ConstVal.CONTROLLER_PATH]) {
                    val controllerFile = String.format(
                            pathInfo[ConstVal.CONTROLLER_PATH].toString() + File.separator + tableInfo.controllerName + suffixJavaOrKt(),
                            entityName
                    )
                    if (isCreate(FileType.CONTROLLER, controllerFile)) {
                        writerFile(objectMap, templateFilePath(template.controller), controllerFile)
                        project.logger.quiet(controllerFile)
                    }
                }
            }
        } catch (e: Exception) {
            logger.error("无法创建文件，请检查配置信息！", e)
        }
        return this
    }
}