/*
 * This Kotlin source file was generated by the Gradle 'init' task.
 */
package com.github.jchanghong

import cn.hutool.core.io.FileUtil
import cn.hutool.core.io.resource.ResourceUtil
import org.gradle.api.Plugin
import org.gradle.api.Project
import org.gradle.api.artifacts.Configuration
import org.gradle.api.tasks.Exec
import org.gradle.api.tasks.testing.Test
import java.io.File

internal fun log2(log: Any?, project: Project, logInfo: Boolean) {
    val get = project.properties.get("jch.debug").toString().toBoolean()
    if (!logInfo && !get) return
    project.logger.quiet("setProject<<${project.name}>>:${log.toString()}")
}

open class JchPluginExtension(val springBootversion: String, val springCloudVersion: String, var kotlinVersion: String) {
    /** org.springframework.boot:spring-boot-starter-*/
    val springBootStarterPrefix = "org.springframework.boot:spring-boot-starter-"

    /** "com/Hik","crypt","vision/apollo","jef",哪些目录是hik的。mavenlocal，用来copy依赖*/
    val hikvsionMavenNameLikeset = mutableSetOf("com/hik", "crypt", "vision/apollo", "jef", "com/pbg","org/postgresql","com/alibaba","net/sourceforge","jxl","org/easyframe","commons-codec")

    /** org.springframework.cloud:spring-cloud-starter- */
    val springCloudStarterPrefix = "org.springframework.cloud:spring-cloud-starter-"

    lateinit var project: Project

    /** add com.github.jchanghong:kotlin-lib*/
    fun addDependenJchanghongutil(version: String = "2.4.0") {
        log2("add dependencies DependenJchanghongutil", project, logInfo)
        project.dependencies.add("implementation", "com.github.jchanghong:kotlin-lib:${version}")
    }

    fun addSpringBootStarter(starter: String, version: String? = null) {
        val version1 = if (version.isNullOrBlank()) springBootversion else version
        log2("add dependencies ${springBootStarterPrefix}${starter}:${version1}", project, logInfo)
        project.dependencies.add("implementation", "${springBootStarterPrefix}${starter}:${version1}")
    }

    fun addSpringCloudStarter(starter: String, version: String? = null) {
        val version1 = if (version.isNullOrBlank()) "" else ":${version}"
        log2("add dependencies ${springCloudStarterPrefix}${starter}${version1}", project, logInfo)
        project.dependencies.add("implementation", "${springCloudStarterPrefix}${starter}${version1}")
    }

    /** org.springframework.kafka:spring-kafka*/
    fun addDependenSpringKafka() {
        project.dependencies.add("implementation", "org.springframework.kafka:spring-kafka")
    }

    fun addNpmBuildTask(action: ((Exec) -> Unit)? = null) {
        //调用npm run dev
        val create = project.tasks.create("npmBuild", Exec::class.java)
        create.apply {
            inputs.dir("src")
            inputs.dir("public")
            inputs.file("package.json")
            outputs.dir("dist")
            commandLine("PowerShell", "-Command", "npm run build")
            group = "jch"
            description = "调用npm run build"
        }
        action?.invoke(create)
    }

    var logInfo = false
    var componentDisplayName=""

    /**         <starfish.version>1.7.1.RELEASE</starfish.version>*/
    var hikStarfishtVersion = ""
    var okhttpVersion = "4.9.0"

    /** 强制设置版本号，key是 group:name,或者group*/
    var mavenVersionMap: MutableMap<String, String> = mutableMapOf()


    /** key是 group:name */
    var tobeRemovedDependencies = mutableSetOf("")

    /** Starter for JMS messaging using Apache ActiveMQ */
    val spring_boot_starter_activemq = "activemq"

    /** Starter for using Spring AMQP and Rabbit MQ */
    val spring_boot_starter_amqp = "amqp"

    /** Starter for aspect_oriented programming with Spring AOP and AspectJ */
    val spring_boot_starter_aop = "aop"

    /** Starter for JMS messaging using Apache Artemis */
    val spring_boot_starter_artemis = "artemis"

    /** Starter for using Spring Batch */
    val spring_boot_starter_batch = "batch"

    /** Starter for using Spring Framework’s caching support */
    val spring_boot_starter_cache = "cache"

    /** Starter for using Cassandra distributed database and Spring Data Cassandra */
    val spring_boot_starter_data_cassandra = "data-cassandra"

    /** Starter for using Cassandra distributed database and Spring Data Cassandra Reactive */
    val spring_boot_starter_data_cassandra_reactive = "data-cassandra-reactive"

    /** Starter for using Couchbase document_oriented database and Spring Data Couchbase */
    val spring_boot_starter_data_couchbase = "data-couchbase"

    /** Starter for using Couchbase document_oriented database and Spring Data Couchbase Reactive */
    val spring_boot_starter_data_couchbase_reactive = "data-couchbase-reactive"

    /** Starter for using Elasticsearch search and analytics engine and Spring Data Elasticsearch */
    val spring_boot_starter_data_elasticsearch = "data-elasticsearch"

    /** Starter for using Spring Data JDBC */
    val spring_boot_starter_data_jdbc = "data-jdbc"

    /** Starter for using Spring Data JPA with Hibernate */
    val spring_boot_starter_data_jpa = "data-jpa"

    /** Starter for using Spring Data LDAP */
    val spring_boot_starter_data_ldap = "data-ldap"

    /** Starter for using MongoDB document_oriented database and Spring Data MongoDB */
    val spring_boot_starter_data_mongodb = "data-mongodb"

    /** Starter for using MongoDB document_oriented database and Spring Data MongoDB Reactive */
    val spring_boot_starter_data_mongodb_reactive = "data-mongodb-reactive"

    /** Starter for using Neo4j graph database and Spring Data Neo4j */
    val spring_boot_starter_data_neo4j = "data-neo4j"

    /** Starter for using Spring Data R2DBC */
    val spring_boot_starter_data_r2dbc = "data-r2dbc"

    /** Starter for using Redis key_value data store with Spring Data Redis and the Lettuce client */
    val spring_boot_starter_data_redis = "data-redis"

    /** Starter for using Redis key_value data store with Spring Data Redis reactive and the Lettuce client */
    val spring_boot_starter_data_redis_reactive = "data-redis-reactive"

    /** Starter for exposing Spring Data repositories over REST using Spring Data REST */
    val spring_boot_starter_data_rest = "data-rest"

    /** Starter for using the Apache Solr search platform with Spring Data Solr */
    val spring_boot_starter_data_solr = "data-solr"

    /** Starter for building MVC web applications using FreeMarker views */
    val spring_boot_starter_freemarker = "freemarker"

    /** Starter for building MVC web applications using Groovy Templates views */
    val spring_boot_starter_groovy_templates = "groovy-templates"

    /** Starter for building hypermedia_based RESTful web application with Spring MVC and Spring HATEOAS */
    val spring_boot_starter_hateoas = "hateoas"

    /** Starter for using Spring Integration */
    val spring_boot_starter_integration = "integration"

    /** Starter for using JDBC with the HikariCP connection pool */
    val spring_boot_starter_jdbc = "jdbc"

    /** Starter for building RESTful web applications using JAX_RS and Jersey. An alternative to spring_boot_starter_web */
    val spring_boot_starter_jersey = "jersey"

    /** Starter for using jOOQ to access SQL databases. An alternative to spring_boot_starter_data_jpa or spring_boot_starter_jdbc */
    val spring_boot_starter_jooq = "jooq"

    /** Starter for reading and writing json */
    val spring_boot_starter_json = "json"

    /** Starter for JTA transactions using Atomikos */
    val spring_boot_starter_jta_atomikos = "jta-atomikos"

    /** Starter for JTA transactions using Bitronix. Deprecated since 2.3.0 */
    val spring_boot_starter_jta_bitronix = "jta-bitronix"

    /** Starter for using Java Mail and Spring Framework’s email sending support */
    val spring_boot_starter_mail = "mail"

    /** Starter for building web applications using Mustache views */
    val spring_boot_starter_mustache = "mustache"

    /** Starter for using Spring Security’s OAuth2/OpenID Connect client features */
    val spring_boot_starter_oauth2_client = "oauth2-client"

    /** Starter for using Spring Security’s OAuth2 resource server features */
    val spring_boot_starter_oauth2_resource_server = "oauth2-resource-server"

    /** Starter for using the Quartz scheduler */
    val spring_boot_starter_quartz = "quartz"

    /** Starter for building RSocket clients and servers */
    val spring_boot_starter_rsocket = "rsocket"

    /** Starter for using Spring Security */
    val spring_boot_starter_security = "security"

    /** Starter for testing Spring Boot applications with libraries including JUnit, Hamcrest and Mockito */
    val spring_boot_starter_test = "test"

    /** Starter for building MVC web applications using Thymeleaf views */
    val spring_boot_starter_thymeleaf = "thymeleaf"

    /** Starter for using Java Bean Validation with Hibernate Validator */
    val spring_boot_starter_validation = "validation"

    /** Starter for building web, including RESTful, applications using Spring MVC. Uses Tomcat as the default embedded container */
    val spring_boot_starter_web = "web"

    /** Starter for using Spring Web Services */
    val spring_boot_starter_web_services = "web-services"

    /** Starter for building WebFlux applications using Spring Framework’s Reactive Web support */
    val spring_boot_starter_webflux = "webflux"

    /** Starter for building WebSocket applications using Spring Framework’s WebSocket support */
    val spring_boot_starter_websocket = "websocket"

    /**Starter for using Spring Boot’s Actuator which provides production ready features to help you monitor and manage your application*/
    val spring_boot_starter_actuator = "actuator"

    //    cloud
    val spring_cloud_starter_consul_config = "consul-config"
    val spring_cloud_starter_consul_discovery = "consul-discovery"
    val spring_cloud_starter_gateway = "gateway"
    val spring_cloud_starter_loadbalancer = "loadbalancer"
    val spring_cloud_starter_openfeign = "openfeign"
}

/**
 * A simple 'hello world' plugin.
 * 一个工程一个对象实例
 */
class JchGradlePlugin : Plugin<Project> {
    companion object {
        val gradleproperties = ResourceUtil.readUtf8Str("gradle.properties")
    }

    lateinit var myExtension: JchPluginExtension
    override fun apply(project: Project) {
        log2(".\\gradlew.bat wrapper --gradle-version=6.8.3", project, true)
        createPorpertiesFile(project)
        myExtension = project.extensions.create("jch", JchPluginExtension::class.java, jchspringBootVersion(project), jchspringCloudVersion(project), jchKotlinVersion(project))
        myExtension.project = project
        setPropertie(project, myExtension)
        setRepositories(project, myExtension)
        addPlugin(project, myExtension)
        project.configurations.all {
            when (it.name) {
                "annotationProcessor" -> annotationProcessor(project, myExtension)
                "compileOnly" -> compileOnly(project, myExtension)
                "developmentOnly" -> developmentOnly(project, myExtension)
                "implementation" -> implementation(project, myExtension)
                "testImplementation" -> testImplementation(project, myExtension)
            }
        }
        addMyTasks(project, myExtension)
        project.afterEvaluate { afterEvaluate(it) }
    }

    private fun compileOnly(project: Project, myExtension: JchPluginExtension) {
        log2("compileOnly()", project, myExtension.logInfo)
        val findByName: Configuration? = project.configurations.findByName("compileOnly")
        val findByName1: Configuration? = project.configurations.findByName("annotationProcessor")
        if (findByName != null && findByName1 != null) {
            findByName.extendsFrom(findByName1)
        }
        project.dependencies.add("implementation", "org.projectlombok:lombok:1.18.16")
    }

    private fun developmentOnly(project: Project, myExtension: JchPluginExtension) {
        log2("developmentOnly()", project, myExtension.logInfo)
        project.pluginManager.withPlugin("org.springframework.boot") {
            project.dependencies.add("developmentOnly", "org.springframework.boot:spring-boot-devtools")
        }

    }

    private fun annotationProcessor(project: Project, myExtension: JchPluginExtension) {
        log2("annotationProcessor()", project, myExtension.logInfo)
        val findByName: Configuration? = project.configurations.findByName("compileOnly")
        val findByName1: Configuration? = project.configurations.findByName("annotationProcessor")
        if (findByName != null && findByName1 != null) {
            findByName.extendsFrom(findByName1)
        }
        project.dependencies.add("annotationProcessor", "org.projectlombok:lombok:1.18.16")
        project.pluginManager.withPlugin("org.springframework.boot") {
            project.dependencies.add("annotationProcessor", "org.springframework.boot:spring-boot-configuration-processor")

        }

    }

    private fun testImplementation(project: Project, myExtension: JchPluginExtension) {
        log2("testImplementation()", project, myExtension.logInfo)
        project.pluginManager.withPlugin("org.springframework.boot") {
            project.tasks.withType(Test::class.java).configureEach {
                it.useJUnitPlatform()
            }
            addtestImplementation("org.springframework.boot:spring-boot-starter-test", myExtension.springBootversion, project)
            log2("add spring-boot-starter-test", project, myExtension.logInfo)
        }
    }

    private fun implementation(project: Project, myExtension: JchPluginExtension) {
        log2("implementation()", project, myExtension.logInfo)
        project.dependencies.add("implementation", project.dependencies
                .platform("org.springframework.cloud:spring-cloud-dependencies:${myExtension.springCloudVersion}"))
        project.dependencies.add("implementation", project.dependencies
                .platform("org.springframework.boot:spring-boot-dependencies:${myExtension.springBootversion}"))
    }

    private fun createPorpertiesFile(project: Project) {
        val file = File(project.rootProject.rootDir, "gradle.properties")
        if (file.exists()) return
        FileUtil.writeUtf8String(gradleproperties, file)
        log2("create file ${file.absolutePath}", project, true)
    }

    private fun afterEvaluate(project: Project) {
        log2("afterEvaluate()", project, myExtension.logInfo)
        if (!project.pluginManager.hasPlugin("org.springframework.boot")) {
            addtestImplementation("org.jetbrains.kotlin:kotlin-test-junit", myExtension.kotlinVersion, project)
            log2("remove web addkotlin test", project, myExtension.logInfo)
        }
        setdependenciesAfterUser(project, myExtension)
    }


}
