package com.github.jchanghong.hikcomponent

import cn.hutool.core.io.FileUtil
import cn.hutool.core.io.resource.ResourceUtil
import com.github.jchanghong.JchPluginExtension
import org.gradle.api.Project
import org.springframework.boot.gradle.tasks.bundling.BootJar
import java.io.File

object HikComponent {
    fun createHikComponent(project: Project, outDirs: String, myExtension: JchPluginExtension): Unit {
        val bootJar = project.tasks.withType(BootJar::class.java).firstOrNull()?.outputs?.files?.firstOrNull()
        if (bootJar == null || !bootJar.exists()) {
            project.logger.quiet("no spring bootJar tasks")
            return
        }

        val outDir = File(outDirs)
        FileUtil.del(outDir)
        FileUtil.mkdir(outDir)
        var pwd = outDir//当前工作目录
        listOf(
                "bin",
                "conf",
                "data",
                "document",
                "download",
                "language",
                "logs",
                "META-INF",
                "resource",
                "script",
                "tool"
        )
                .forEach { FileUtil.mkdir(File(pwd, it)) }
        pwd = File(outDir, "bin")
        listOf("jdbcdb", "mqqueue", "rediscache", "remoteconsul", "webservice")
                .forEach { FileUtil.mkdir(File(pwd, it)) }
        pwd = File(outDir, "script")
        listOf("jdbcdb", "mqqueue", "rediscache", "remoteconsul", "webservice")
                .forEach { FileUtil.mkdir(File(pwd, it)) }

        pwd = File(outDir, "language")
        listOf("en_US", "zh_CN")
                .forEach { FileUtil.mkdir(File(pwd, it)) }
        pwd = File(outDir, "META-INF")
        listOf("icon", "language")
                .forEach { FileUtil.mkdir(File(pwd, it)) }

        pwd = File(outDir, "META-INF/icon")
        listOf("download", "menu")
                .forEach { FileUtil.mkdir(File(pwd, it)) }
        pwd = File(outDir, "META-INF/language")
        listOf("en_US", "zh_CN")
                .forEach { FileUtil.mkdir(File(pwd, it)) }
//        下面是生成文件
        copyFileToOut("bin/jdbcdb/init.sql", project, outDir)
        copyFileToOut("language/en_US/translate.properties", project, outDir)
        copyFileToOut("language/zh_CN/translate.properties", project, outDir)
        listOf(
                "META-INF/language/en_US/device_support.txt",
                "META-INF/language/en_US/function_description.txt",
                "META-INF/language/en_US/performance_description.txt",
                "META-INF/language/en_US/changelog.txt",
                "META-INF/language/en_US/translate.properties",
                "META-INF/language/zh_CN/device_support.txt",
                "META-INF/language/zh_CN/function_description.txt",
                "META-INF/language/zh_CN/performance_description.txt",
                "META-INF/language/zh_CN/changelog.txt",
                "META-INF/language/zh_CN/translate.properties",
                "META-INF/component.xml",
                "META-INF/config.xml",
                "META-INF/documentinfo.xml",
                "META-INF/downloadinfo.xml",
                "META-INF/features.xml",
                "META-INF/file_checksum.xml",
                "META-INF/installation.xml",
                "META-INF/licensenotice.xml",
                "META-INF/menus.xml",
                "META-INF/monitor.xml",
                "META-INF/packageinfo.xml",
                "META-INF/program_checksum.xml",
                "META-INF/properties.xml",
                "META-INF/versioninfo.xml",
                "script/webservice/postinstall.sh",
                "script/webservice/postuninstall.sh",
                "script/webservice/start.sh",
                "script/webservice/stop.sh",
                "script/webservice/service"
        ).forEach { copyFileToOut(it, project, outDir) }
        FileUtil.copy(bootJar.absolutePath, "${outDirs}/bin/webservice", true)
        FileUtils.replaceInDir(project, outDir, "webservice.jar", bootJar.name)
        val componentid = project.findProperty("jch.componentname")?.toString() ?: ""
        val displayName = myExtension.componentDisplayName
        if (componentid.isBlank()) {
            project.logger.quiet("jch.componentname Property not exit use 属性为空， 组件名默认为 componentname")
        }
        else{
            FileUtils.replaceInDir(project, outDir, "componentname", componentid)
        }
        if (displayName.isBlank()) {
            project.logger.quiet("jch.componentDisplayName Property not exit use 属性为空， 组件名默认为 gradle自动生成组件")

        }
        else{
            FileUtils.replaceInDir(project, outDir, "gradle自动生成组件", displayName)
        }
        if (!componentid.isBlank()) {
//            更新目录名称
            val replace = outDir.absolutePath.replace("componentname", componentid)
            project.logger.quiet("更新目录名称 $replace")
            FileUtil.rename(outDir,replace,true)
        }
    }

    private fun copyFileToOut(file: String, project: Project, outDir: File) {
        val d = File(outDir, file)
        val stream = ResourceUtil.getStream("componentname_1.0.100.20200408205737/$file")
        d.writeBytes(stream.readBytes())
    }

}