package com.github.jchanghong

import org.gradle.api.Project
import org.gradle.api.internal.artifacts.dependencies.DefaultExternalModuleDependency
import org.gradle.api.tasks.testing.Test

/** 第一个是group map，第2个是 完整的group:name map*/
fun versions(map: Map<String, String>): Pair<Map<String, String>, Map<String, String>> {
    val groupmap = map.filterKeys { it.indexOf(":") < 0 }
    val groupmap2 = map.filterKeys { it.indexOf(":") > 0 }
    return groupmap to groupmap2
}

internal fun setdependenciesAfterUser(project: Project, myExtension: JchPluginExtension) {
    log2("setdependenciesAfterUser()", project, myExtension.logInfo)
    val (groupVersionMap, groupNameVersionMap) = versions(myExtension.mavenVersionMap)
    project.configurations.all {
        it.resolutionStrategy.eachDependency {
            val key = "${it.requested.group}:${it.requested.name}"
            if (it.requested.group == "org.jetbrains.kotlin") {
                it.useVersion(myExtension.kotlinVersion)
            }
            if (it.requested.group == "com.squareup.okhttp3" && it.requested.name == "okhttp") {
                it.useVersion(myExtension.okhttpVersion)
            }
            if (groupVersionMap.containsKey(it.requested.group)) {
                it.useVersion(groupVersionMap[it.requested.group] ?: error(""))
            }
            if (groupNameVersionMap.containsKey(key)) {
                it.useVersion(groupNameVersionMap[key] ?: error(""))
            }
            if (key in myExtension.tobeRemovedDependencies) {
                it.useTarget("org.jetbrains.kotlin:kotlin-stdlib-common:${myExtension.kotlinVersion}")
            }
        }
    }
}

internal fun setdependenciesBeforUser(project: Project, myExtension: JchPluginExtension) {
    log2("setdependenciesBeforUser()", project, myExtension.logInfo)
    project.configurations.findByName("compileOnly")?.extendsFrom(
            project.configurations.findByName("annotationProcessor")
    )
    project.dependencies.add("annotationProcessor", "org.projectlombok:lombok")
    project.dependencies.add("compileOnly", "org.projectlombok:lombok")

    if (project.pluginManager.hasPlugin("org.springframework.boot")) {
        project.tasks.withType(Test::class.java).configureEach {
            it.useJUnitPlatform()
        }
        project.dependencies.add("developmentOnly", "org.springframework.boot:spring-boot-devtools")
        project.dependencies.add("annotationProcessor", "org.springframework.boot:spring-boot-configuration-processor")
        log2("add spring-boot-devtools", project, myExtension.logInfo)
        addtestImplementation(
                "org.springframework.boot:spring-boot-starter-test",
                myExtension.springBootversion,
                project
        )
                ?.exclude(mapOf("group" to "org.junit.vintage", "module" to "junit-vintage-engine"))
        log2("add spring-boot-starter-test", project, myExtension.logInfo)
    } else {
        addtestImplementation("org.jetbrains.kotlin:kotlin-test-junit", myExtension.kotlinVersion, project)
        log2("remove web", project, myExtension.logInfo)
    }
}

internal fun addtestImplementation(
        key: String,
        version: String,
        project: Project,
): DefaultExternalModuleDependency? {
    val add = project.dependencies.add("testImplementation", "${key}:${version}")
    return (add as? DefaultExternalModuleDependency?)
}